/**
 * Created on 04.03.2008
 */
package lab2.data;

import java.util.ArrayList;
import java.util.List;

import lab2.types.Airport;
import lab2.types.Flight;
import lab2.types.MyClass;

/**
 * @author cipi
 * 
 */
public class ProjectCharacteristics extends MyClass {
	public static final String AIRPLANE_TO_GO = "Airplane to go";
	public static final String AIRPLANE_LANDED = "Airplane landed";
	public static final String UPDATE = "Update";
	
	private static ProjectCharacteristics instance = null;

	private List<Airport> airports = new ArrayList<Airport>();
	private List<Flight> flightsInAir = new ArrayList<Flight>();
	private List<Flight> flightsWaiting = new ArrayList<Flight>();

	private ProjectCharacteristics() {
		airports = DefaultData.getAirports();
	}

	public static ProjectCharacteristics getInstance() {
		if (instance == null) {
			instance = new ProjectCharacteristics();
		}

		return instance;
	}

	public List<Airport> getAirports() {
		return airports;
	}

	public void addAirport(Airport airport) {
		airports.add(airport);
	}

	public void removeAirport(Airport airport) {
		airports.remove(airport);
	}

	public List<Flight> getFlightsInAir() {
		//it would be safer to offer a copy
		return flightsInAir;
	}
	
	private void removeFlightInAir(Flight flight) {
		flightsInAir.remove(flight);
		notifyListeners(AIRPLANE_LANDED);
	}

	public List<Flight> getFlightsWaiting() {
		return flightsWaiting;
	}
	
	public void addFlightWaiting(Flight flight) {
		flightsWaiting.add(flight);
		notifyListeners(AIRPLANE_TO_GO);
	}
	
	private void startFlight(Flight flight) {
		if(!flightsWaiting.remove(flight)) {
			throw new IllegalArgumentException("Tried to start a flight "
					+ "which was not waiting");
		}
		flight.startFlying();
		flightsInAir.add(flight);
	}
	
	public void updateFlights() {
		List<Flight> flightsToBeStarted = new ArrayList<Flight>();
		List<Flight> flightsArrived = new ArrayList<Flight>();
		
		for (Flight flight : flightsWaiting) {
			if(canStart(flight)) {
				flightsToBeStarted.add(flight);
			}
		}
		
		for (Flight flight : flightsToBeStarted) {
			startFlight(flight);
		}
		
		for (Flight flight : flightsInAir) {
			flight.advance();
			if(!flight.isFlying()) {
				flightsArrived.add(flight);
			}
		}
		
		for (Flight flight : flightsArrived) {
			removeFlightInAir(flight);
		}
		
		notifyListeners(UPDATE);
	}
	
	private boolean canStart(Flight flightWaiting) {
		for (Flight flightInAir : flightsInAir) {
			if(flightInAir.collides(flightWaiting)) {
				return false;
			}
		}
		
		return true;
	}

}
