/**
 * Created on 08.03.2008
 */
package lab2.gui.airplanes;

import java.text.DecimalFormat;

import javax.swing.table.AbstractTableModel;

import lab2.data.ProjectCharacteristics;
import lab2.types.Flight;
import lab2.types.MyClass;
import lab2.types.MyEventListener;

/**
 * @author cipi
 *
 */
@SuppressWarnings("serial")
public class AirplanesTableModel extends AbstractTableModel {
	private static final String[] TABLE_COL_NAMES = { "Tip avion",
			"Aeroportul de plecare", "Aeroportul destinatie",
			"<html><Center>Distanta<br>totala</html>",
			"<html><Center>Distanta pana<br>la destinatie</html>",
			"Coordonatele actuale", "Status" };
	
	DecimalFormat decimalFormat = null;
	private ProjectCharacteristics characteristics = null;
	
	public AirplanesTableModel() {
		characteristics = ProjectCharacteristics.getInstance();
		decimalFormat = new DecimalFormat("#.#");
		addListeners();
	}
	
	public int getColumnCount() {
		return TABLE_COL_NAMES.length;
	}
	
	@Override
	public String getColumnName(int col) {
		return TABLE_COL_NAMES[col];
	}

	public int getRowCount() {
		return characteristics.getFlightsWaiting().size()
				+ characteristics.getFlightsInAir().size();
	}

	public Object getValueAt(int row, int col) {
		Flight flight = null;
		int airplanesWaiting = characteristics.getFlightsWaiting().size();
		
		if(row < airplanesWaiting) {
			flight = characteristics.getFlightsWaiting().get(row);
		} else {
			flight = characteristics.getFlightsInAir().get(row - airplanesWaiting);
		}
		
		switch (col) {
		case 0:
			return "" + flight.getAirplane();	
		case 1:
			return "" + flight.getSource();
		case 2:
			return "" + flight.getDestination();
		case 3:
			return decimalFormat.format(flight.getWholeDistance());
		case 4:
			return decimalFormat.format(flight.getDistanceToTheTarget());
		case 5:
			return "" + flight.getAirplaneCoordinates();
		case 6: 
			return flight.isFlying()? "Zboara" : "Asteapta";
		default:
			return "TODO";
		}
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}
	
	private void addListeners() {
		characteristics.addMyEventListener(new MyEventListener() {

			public void performMyEvent(MyClass sender, String evt) {
				if (evt.equals(ProjectCharacteristics.AIRPLANE_TO_GO)
						|| evt.equals(ProjectCharacteristics.AIRPLANE_LANDED)) {
					fireTableStructureChanged();
				}
				if (evt.equals(ProjectCharacteristics.UPDATE)) {
					fireTableDataChanged();
				}
			}

		});
	}

}
