/**
 * Created on 08.03.2008
 */
package lab2.gui.airports;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.KeyStroke;

import lab2.data.ProjectCharacteristics;
import lab2.types.Airplane;
import lab2.types.Airport;
import lab2.types.Flight;

/**
 * @author cipi
 *
 */
@SuppressWarnings("serial")
public class AirplanesDepartDialog extends JDialog {
	private JLabel label;
	private JComboBox comboBox = null;
	private JButton okBtn = null;
	private JButton cancelBtn = null;
	private Color bkColor = null;
	
	private ActionListener actionListener = null;
	
	private Airplane airplane = null;
	private Airport airport = null;
	private ProjectCharacteristics characteristics = null;

	public AirplanesDepartDialog(Airport airport, Airplane airplane, JFrame frame) {
		super(frame, true);
		this.airport = airport;
		this.airplane = airplane;
		initComponents();
		initialize();
		setLocationRelativeTo(frame);
	}

	private void initComponents() {
		setResizable(false);
		setTitle("Decolarea avionului " + airplane);
		
		characteristics = ProjectCharacteristics.getInstance();
		
		label = new JLabel("Aeroportul spre care doriti sa plece: ");
		
		comboBox = new JComboBox(characteristics.getAirports().toArray());
		comboBox.removeItem(airport);
		
		actionListener = new ActionListener() {
			public void actionPerformed(ActionEvent actionEvent) {
				dispose();
			}
		};
		
		okBtn = new JButton("    Ok    ");
		okBtn.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				dispose();
				characteristics.addFlightWaiting(new Flight(airport,
						(Airport) comboBox.getSelectedItem(), airplane));
			}		
		});
		cancelBtn = new JButton("Cancel");
		cancelBtn.addActionListener(actionListener);
		
		KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
		rootPane.registerKeyboardAction(actionListener, stroke, JComponent.WHEN_IN_FOCUSED_WINDOW);
		
		bkColor = new Color(185, 239, 224);
	}

	private void initialize() {
		JPanel panel = new JPanel();
		panel.setLayout(new GridBagLayout());
		panel.setBorder(null);
		panel.setOpaque(false);
		
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.CENTER;
		c.insets = new Insets(0, 0, 0, 10);
		c.gridy = 0;
		c.gridx = 0;
		
		panel.add(label, c);
		c.gridx++;
		c.insets = new Insets(0, 0, 0, 0);
		panel.add(comboBox, c);
		
		JPanel buttonsPanel = new JPanel();
		buttonsPanel.setLayout(new BoxLayout(buttonsPanel, BoxLayout.X_AXIS));
		buttonsPanel.setBorder(null);
		buttonsPanel.setOpaque(false);
		buttonsPanel.add(okBtn);
		buttonsPanel.add(Box.createHorizontalStrut(10));
		buttonsPanel.add(cancelBtn);	
		
		getContentPane().setLayout(new GridBagLayout());
		c.insets = new Insets(10, 10, 10, 10);
		c.gridy = 0;
		c.gridx = 0;
		
		getContentPane().add(panel, c);
		
		c.anchor = GridBagConstraints.LINE_END;
		c.gridy++;
		getContentPane().add(buttonsPanel, c);
		
		getContentPane().setBackground(bkColor);
		comboBox.setBackground(bkColor);
		
		pack();
	}

}
