/**
 * Created on 08.03.2008
 */
package lab2.gui.airports;

import javax.swing.table.AbstractTableModel;

import lab2.data.ProjectCharacteristics;
import lab2.types.Airport;
import lab2.types.MyClass;
import lab2.types.MyEventListener;

/**
 * @author cipi
 *
 */
@SuppressWarnings("serial")
public class AirportsTableModel extends AbstractTableModel {
	private ProjectCharacteristics characteristics = null;
	
	public AirportsTableModel() {
		characteristics = ProjectCharacteristics.getInstance();
		addListeners();
	}

	public int getColumnCount() {
		return characteristics.getAirports().size();
	}
	
	@Override
	public String getColumnName(int col) {
		return "" + characteristics.getAirports().get(col);
	}

	public int getRowCount() {
		int max = 0;
		for (Airport airport : characteristics.getAirports()) {
			max = Math.max(max, airport.getAirplanes().size());
		}
		return max;
	}

	public Object getValueAt(int row, int col) {
		Airport airport = characteristics.getAirports().get(col);
		
		if(airport.getAirplanes().size() > row) {
			return airport.getAirplanes().get(row);
		}
		
		return " ";
	}
	
	@Override
	public boolean isCellEditable(int row, int col) {
		return row < characteristics.getAirports().get(col).getAirplanes().size();
	}

	private void addListeners() {
		for (Airport airport : characteristics.getAirports()) {
			airport.addMyEventListener(new MyEventListener() {

				public void performMyEvent(MyClass sender, String evt) {
					if (evt.equals(Airport.AIRPLANE_ARRIVED)
							|| evt.equals(Airport.AIRPLANE_LEAVED)) {
						fireTableStructureChanged();
					}
				}
				
			});
		}
	}
}
