/**
 * Created on 05.03.2008
 */
package lab2.types;

/**
 * @author cipi
 *
 */
public class Flight {
	public static final int step = 10;
	public static final int minimumSafetyDistance = 2;
	
	private Airport source = null;
	private Airport destination = null;
	private Airplane airplane = null;
	private Coordinates airplaneCoordinates = null;
	private boolean flying;
	private int direction; //should be allways 1 or -1
	private double slope = Double.MAX_VALUE;
	private double ordinateAtOrigin;
	private double wholeDistance;
	private double sx; //the abscisa of the source airport
	private double sy; //the ordinate of the source airport
	private double dx; //the abscisa of the destination airport
	private double dy; //the ordinate of the destination airport

	public Flight(Airport source, Airport destination, Airplane airplane) {
		this.source = source;
		this.destination = destination;
		this.airplane = airplane;
		
		if (!source.containsAirplane(airplane)) {
			throw new IllegalArgumentException("The airport '" + source
					+ "' doesn't contain the airplane '" + airplane + "'!!!");
		}

		initialize();
	}
	
	private void initialize() {
		airplaneCoordinates = source.getAirportInformation()
				.getAirportCoordinates();
		flying = false;
		wholeDistance = computeTheDistance(source.getAirportInformation()
				.getAirportCoordinates(), destination.getAirportInformation()
				.getAirportCoordinates());
		
		sx = source.getAirportInformation().getAirportCoordinates().getX();
		sy = source.getAirportInformation().getAirportCoordinates().getY();
		dx = destination.getAirportInformation().getAirportCoordinates().getX();
		dy = destination.getAirportInformation().getAirportCoordinates().getY();
		
		if ((sx == dx) && (sy == dy)) {
			throw new IllegalArgumentException(
					"The coordinates of the two airports " + " are identical!!!");
		}
		
		if(sx == dx) { // "vertical" flight, slope not computable
			// another approach with Math.signum could be implemented
			if (sy < dy) {
				// flies from down to up
				direction = 1;
			} else {
				direction = -1;
			}
		} else {
			slope = (dy - sy) / (dx - sx);
			ordinateAtOrigin = sy - slope * sx;

			if (sx < dx) {
				// flies from the left to the right
				direction = 1;
			} else {
				direction = -1;
			}
		}
		
		source.removeAirplane(airplane); // this way will be on hold
	}

	private double computeTheDistance(Coordinates sourceCoordinates,
			Coordinates destCoordinates) {
		double x = sourceCoordinates.getX() - destCoordinates.getX();
		double y = sourceCoordinates.getY() - destCoordinates.getY();
		
		return Math.sqrt(x*x + y*y);
	}

	/**
	 *	this method is used to take off the plain
	 */
	public void startFlying() {
		this.flying = true;
	}

	/**
	 *	using this method you advance the plain one more step
	 *	to the destination
	 */
	public void advance() {
		if(isFlying()) {
			if (computeTheDistance(airplaneCoordinates, destination
					.getAirportInformation().getAirportCoordinates()) > step) {
				setTheNewAirplaneCoordinates();
			} else {
				destination.addAirplane(airplane);
				airplaneCoordinates = destination.getAirportInformation()
						.getAirportCoordinates();
				flying = false;
			}
		}
	}

	private void setTheNewAirplaneCoordinates() {
		if(sy == dy) {
			airplaneCoordinates = new Coordinates(airplaneCoordinates.getX()
					+ step * direction, airplaneCoordinates.getY());
		} else if (sx == dx) {
			airplaneCoordinates = new Coordinates(airplaneCoordinates.getX(),
					airplaneCoordinates.getY() + step * direction);
		} else { // do it on paper first if you don't believe me :P
			double newX = airplaneCoordinates.getX()
					+ direction * Math.sqrt(step * step / (1 + slope * slope));
			double newY = slope * newX + ordinateAtOrigin;
			airplaneCoordinates = new Coordinates(newX, newY);
		}
	}

	public Airplane getAirplane() {
		return airplane;
	}

	/**
	 * @return the airplane coordinates at this moment
	 */
	public Coordinates getAirplaneCoordinates() {
		return airplaneCoordinates;
	}

	/**
	 * @return the arrival airport
	 */
	public Airport getDestination() {
		return destination;
	}

	/**
	 * @return if the airplane is flying or not
	 */
	public boolean isFlying() {
		return flying;
	}

	/**
	 * @return the depart airport
	 */
	public Airport getSource() {
		return source;
	}

	/**
	 * @return the distance between the two airports
	 */
	public double getWholeDistance() {
		return wholeDistance;
	}
	
	/**
	 * @return the distance the plain already travelled
	 */
	public double getTrackedDistance() {
		return computeTheDistance(source.getAirportInformation()
				.getAirportCoordinates(), airplaneCoordinates);
	}

	/**
	 * @return the distance the plain must travel to reach the destination
	 */
	public double getDistanceToTheTarget() {
		return computeTheDistance(airplaneCoordinates, destination
				.getAirportInformation().getAirportCoordinates());
	}
	
	/**
	 * @return the flight slope (the tangenta of the angle between
	 * 		the flight right and the abscisa right)
	 */
	public double getSlope() {
		return slope;
	}

	public double getOrdinateAtOrigin() {
		return ordinateAtOrigin;
	}

	/**
	 * @return true if this flight will collide with the given flight 
	 */
	public boolean collides(Flight otherFlight) {
		double apX = airplaneCoordinates.getX();
		double apY = airplaneCoordinates.getY();
		double ofApX = otherFlight.getAirplaneCoordinates().getX();
		double ofApY = otherFlight.getAirplaneCoordinates().getY();
		double ofDestX = otherFlight.getDestination().getAirportInformation()
				.getAirportCoordinates().getX();
		double ofDestY = otherFlight.getDestination().getAirportInformation()
				.getAirportCoordinates().getY();

		if (slope == otherFlight.getSlope()) {
			// the support right of the 2 flights are identical or ||
			if (ordinateAtOrigin == otherFlight.getOrdinateAtOrigin()) {
				// the support right of the 2 flights are identical
				if (apX == dx) {
					// the trajectory is |
					if ((Math.min(apY, dy) < Math.max(ofApY, ofDestY))
							// put <= if 2 airplaines can't land
							// in the same time on one airport
							&& (Math.min(ofApY, ofDestY) < Math.max(apY, dy))) {
						// The trajectories intersects
						// But the collision depends on the airplanes position
						if ((!between(apY, dy, ofDestY))
								&& (!between(ofApY, dy, ofDestY))) {
							//any airplaine isn't between the airports
							if(between(dy, apY, ofApY)) {
								//the airports are between the airplains
								if(!oneFlightOversBeforeOtherPlainReaches(otherFlight)) {
									//neither one airplane will reach it's destination
									//before the other one reaches
									return true;
								}
							} // else the plains will go one after another
						} else if((!between(dy, apY, ofApY))
								&& (!between(ofDestY, apY, ofApY))) {
							//any airport isn't between the airplanes
							//(but the airplanes are between the airports)
							return true;
						} else {
							// one aiplane is between the airports
							// one airport is between the airplanes
							// the airplanes are heading to each other
							if(!oneFlightOversBeforeOtherPlainReaches(otherFlight)) {
								//neither one airplane will reach it's destination
								//before the other one reaches
								return true;
							}
						}
					}
				} else {
					// the trajectory is -- or /
					if ((Math.min(apX, dx) < Math.max(ofApX, ofDestX))
							&& (Math.min(ofApX, ofDestX) < Math.max(apX, dx))) {
						// The trajectories intersects
						// But the collision depends on the airplanes position
						if ((!between(apX, dx, ofDestX))
								&& (!between(ofApX, dx, ofDestX))) {
							//any airplaine isn't between the airports
							if(between(dx, apX, ofApX)) {
								//the airports are between the airplains
								if(!oneFlightOversBeforeOtherPlainReaches(otherFlight)) {
									//neither one airplane will reach it's destination
									//before the other one reaches
									return true;
								}
							} // else the plains will go one after another
						} else if((!between(dx, apX, ofApX))
								&& (!between(ofDestX, apX, ofApX))) {
							//any airport isn't between the airplanes
							//(but the airplanes are between the airports)
							return true;
						} else {
							// one aiplane is between the airports
							// one airport is between the airplanes
							// the airplanes are heading to each other
							if(!oneFlightOversBeforeOtherPlainReaches(otherFlight)) {
								//neither one airplane will reach it's destination
								//before the other one reaches
								return true;
							}
						}
					} /* the trajectories intersects */
				} /* the trajectory type */
			} /* the trajectories have the same right support */
		} else {
			// the support right of the 2 flights are neither identical or ||

			/* determine their intersection point */
			double ipx; //intersection point abscisa
			double ipy; //intersection point ordinate
			
			if (apX == dx) {
				// this flight trajectory is | => slope is infinity
				ipx = apX;
				ipy = otherFlight.getSlope() * ipx
						+ otherFlight.getOrdinateAtOrigin();
			} else if (ofApX == ofDestX) {
				// the other flight trajectory is | => it's slope is infinity
				ipx = ofApX;
				ipy = slope * ipx + ordinateAtOrigin;
			} else {
				// the both trajectories can't be | because the slopes are different
				ipx = (otherFlight.getOrdinateAtOrigin() - ordinateAtOrigin)
						/ (slope - otherFlight.getSlope());
				ipy = slope * ipx + ordinateAtOrigin;
			}
			
			/* they can intersect only if each flight goes through  
			   the intersection point */
			if (between(ipx, apX, dx) && between(ipy, apY, dy)
					&& between(ipx, ofApX, ofDestX)
					&& between(ipy, ofApY, ofDestY)) {
				Coordinates ipCoordinates = new Coordinates(ipx, ipy);

				return Math.abs(computeTheDistance(airplaneCoordinates,
						ipCoordinates)
						- computeTheDistance(otherFlight
								.getAirplaneCoordinates(), ipCoordinates)) < minimumSafetyDistance;
			}
		}

		return false;
	}

	private boolean between(double nb, double nb1, double nb2) {
		if(nb1 < nb2) {
			return (nb >= nb1) && (nb <= nb2);
		} else {
			return (nb >= nb2) && (nb <= nb1);
		}
	}
	
	/**
	 * used to determine if this flight can collide with a given flight<br>
	 * it is used only if the plains are heading one to the other<br>
	 * the question is if one of them will land before other one arrives
	 * 
	 * @return 	true if this plaine or the other one will reach it's airport  
	 * 			before the other one arrives 
	 */
	private boolean oneFlightOversBeforeOtherPlainReaches(Flight otherFlight) {
		return (getDistanceToTheTarget() <= computeTheDistance(otherFlight
				.getAirplaneCoordinates(), destination.getAirportInformation()
				.getAirportCoordinates()))
				|| (otherFlight.getDistanceToTheTarget() <= computeTheDistance(
						airplaneCoordinates, otherFlight.getDestination()
								.getAirportInformation()
								.getAirportCoordinates()));
	}
}
